/*
 * IBM Confidential
 * OCO Source Materials
 * Component IBM Rational ROSE_RT BUILD : ROSE_RT
 * (c) Copyright IBM Corp. [1998], [2004] .All Rights Reserved.
 * 
 * The source code for this program is not published or
 * or otherwise divested of its trade secrets, irrespective
 * of what has been deposited with the U.S. Copyright Office.
 * 
 * 
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 * 
 */

#ifndef __RTDecoding_h__
#define __RTDecoding_h__ included

#ifdef PRAGMA
#pragma interface
#endif

#ifndef __RTConfig_h__
#include <RTConfig.h>
#endif

#if OBJECT_DECODE

struct RTFieldDescriptor;
struct RTObject_class;

class RTDecoding
{
public:
		RTDecoding( void );
    virtual     ~RTDecoding( void );

    virtual int get( void                 ** data,
		     const RTObject_class ** type ) = 0;

    // basic data types
    virtual int get_address ( void         * & ) = 0;
    virtual int get_bool    ( bool           & ) = 0;
    virtual int get_char    ( char           & ) = 0;
#if RTUseFloatingPoint
    virtual int get_double  ( double         & ) = 0;
    virtual int get_float   ( float          & ) = 0;
#endif
    virtual int get_int     ( int            & ) = 0;
    virtual int get_long    ( long           & ) = 0;
	virtual int get_unsignedint(unsigned int & ) = 0;
	virtual int get_unsignedlong(unsigned long & ) = 0;
	virtual int get_unsignedshort(unsigned short & ) = 0;
	virtual int get_unsignedchar(unsigned char & ) = 0;
#if 0
#if _MSC_VER > 1000
    virtual int get_long_long( __int64 &  )      = 0;
#else
    virtual int get_long_long( long long &  )    = 0;
#endif    
#endif
    virtual int get_short   ( short          & ) = 0;
    virtual int get_uchar   ( unsigned char  & ) = 0;
    virtual int get_unsigned( unsigned       & ) = 0;
    virtual int get_ulong   ( unsigned long  & ) = 0;
    virtual int get_ushort  ( unsigned short & ) = 0;

    // a nul-terminated string
    virtual int get_string( char * & ) = 0;

    // zero or more raw bytes
    virtual int get_opaque( int &, char * & ) = 0;

    // an enumerated value
    virtual int get_enum( int                     & value,
			  int                       numChoices,
			  const RTFieldDescriptor * choices ) = 0;

    // a homogeneous array of objects
    virtual int get_array( void                 *,
			   int                    numElements,
			   const RTObject_class * elementType ) = 0;

    // an indirect object
    virtual int get_indirect( void                 * pointer,
			      const RTObject_class * targetType ) = 0;

    // a structure
    virtual int get_struct( void                 * record,
			    const RTObject_class * recordType ) = 0;

private:
    // unavailable methods
    RTDecoding( const RTDecoding & );
    RTDecoding & operator=( const RTDecoding & );
};
#endif // OBJECT_DECODE

#endif // __RTDecoding_h__
