/* 
 *  Licensed Materials - Property of IBM 
 *  Component IBM Rational ROSE_RT BUILD : TARGET_RTS 
 *  (c) Copyright IBM Corporation 1991, 2006 All Rights Reserved. 
 */

#include <RTIOMonitor.h>
#include <RTMemoryUtil.h>
#include <RTTimespec.h>
#include <RTtcp.h>

#if defined( VXWORKS60 )
#include <selectLib.h>
#endif

int RTIOMonitor::wait( const RTTimespec * timeout )
{
    fd_set rSet;
    fd_set wSet;
    fd_set eSet;

    RTMemoryUtil::memset( &rSet, 0, (int)sizeof( rSet ) );
    RTMemoryUtil::memset( &wSet, 0, (int)sizeof( wSet ) );
    RTMemoryUtil::memset( &eSet, 0, (int)sizeof( eSet ) );

    int        max_fd = -1;
    Entry    * entry;
    unsigned   index;

    for( entry = _entries, index = 0U; index < _used; ++entry, ++index )
    {
	unsigned fd   = entry->fd;
	int      bits = entry->bits &= RequestMask;

	if( max_fd < (int)fd )
	    max_fd = (int)fd;

	//lint -save -e647 -e661 -e662 -e909 -e912
	if( ( bits & CanRead   ) != 0 ) FD_SET( fd, &rSet );
	if( ( bits & CanWrite  ) != 0 ) FD_SET( fd, &wSet );
	if( ( bits & Exception ) != 0 ) FD_SET( fd, &eSet );
	//lint -restore
    }

    struct timeval * ptv;
    struct timeval   tv;

    if( timeout == (const RTTimespec *)0 )
	ptv = (struct timeval *)0;
    else
    {
	tv.tv_sec  = timeout->tv_sec;
	tv.tv_usec = timeout->tv_nsec / 1000L;
	ptv = &tv;
    }

    if( (long)select( max_fd + 1, &rSet, &wSet, &eSet, ptv ) <= 0L )
	return 0;

    for( entry = _entries, index = 0U; index < _used; ++entry, ++index )
    {
	unsigned fd   = entry->fd;
	int      bits = 0;

	//lint -save -e647 -e909 -e912
	if( FD_ISSET( fd, &rSet ) ) bits |= ResultCanRead;
	if( FD_ISSET( fd, &wSet ) ) bits |= ResultCanWrite;
	if( FD_ISSET( fd, &eSet ) ) bits |= ResultException;
	//lint -restore

	entry->bits |= bits;
    }

    return 1;
}
