/* 
 *  Licensed Materials - Property of IBM 
 *  Component IBM Rational ROSE_RT BUILD : TARGET_RTS 
 *  (c) Copyright IBM Corporation 1991, 2006 All Rights Reserved. 
 */

#ifdef PRAGMA
#pragma implementation "RTThreadInfo.h"
#endif

#include <RTDiag.h>
#include <RTJob.h>
#include <RTSyncObject.h>
#include <RTThread.h>
#include <RTThreadInfo.h>

// Select taskSpawn declaration which accepts a variable number of arguments.
#define __PROTOTYPE_5_0 1

#include <taskLib.h>

static void thread_run( RTJob * job )
{
    // taskSafe protects this thread from deletion
    taskSafe();

    job->mainLoop();

    // this thread is done: allow taskDelete to proceed
    taskUnsafe();

    for(;;)
	if( taskSuspend( taskIdSelf() ) != OK )
	    RTDiag::panic( "taskSuspend" );
}

RTThread::RTThread( RTJob * job, int stack_size, int prio )
    : _job ( job )
    , _info( new RTThreadInfo )
{
    int options;

    if( taskOptionsGet( taskIdSelf(), &options ) != OK )
	RTDiag::panic( "taskInfoGet" );

    else
    {
	_info->tid = taskSpawn( (char *)job->name(),
				prio,
				options & VX_FP_TASK,
				stack_size,
				(FUNCPTR)thread_run,
				(int)job,0,0,0,0,0,0,0,0,0 );

	if( _info->tid == ERROR )
	    RTDiag::panic( "taskSpawn" );
    }
}

RTThreadInfo::RTThreadInfo( void )
    : tid( ERROR )
{
}

RTThreadInfo::~RTThreadInfo( void )
{
    if( tid == ERROR )
    {
	// never spawned
    }
    else if( taskDelete( tid ) != OK )
    {
	RTDiag::panic( "taskDelete" );
    }
}
