/*
 * Copyright (c) 1991-2003 Rational Software Corporation. All rights
 * reserved.
 * 
 * Rational, the Rational logo, ObjecTime (and logo) are registered
 * trademarks of Rational Software Corporation.
 * 
 * The placing of the copyright notices on Rational source code is not
 * intended to mean that they are published works, but are included in the
 * event of inadvertent publication. The source code of Rational contains
 * trade secrets and other valuable confidential information of Rational
 * Software Corporation. Rational Software Corporation and its licensors
 * retain ownership to Rational Products in both source and object code
 * format, and related documentation and all rights therein including
 * copyrights, patents and trade secrets.
 * 
 * 
 * U.S. Government Rights
 * 
 * Use, duplication, or disclosure by the U.S. Government is subject to
 * restrictions set forth in the applicable Rational License Agreement
 * and in DFARS 227.7202-1(a) and 227.7202-3(a) (1995), DFARS
 * 252.227-7013(c)(1)(ii)(Oct 1988), FAR 12.212(a) 1995, FAR 52.227-19,
 * or FAR 52.227-14, as applicable.
 * 
 * Unless otherwise agreed to in writing, use of Rational products is
 * governed by the Rational Product License Agreement provided with
 * the product.
 */

#ifndef __RTtcp_h__
#define __RTtcp_h__ included

#if defined (VXWORKS60)
#define __INCstringh // avoid including string.h
#endif

#include <errno.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sockLib.h>
#include <hostLib.h>
#include <ioLib.h>
#include <net/if.h>

typedef struct sockaddr  SocketAddress;
typedef int              SocketAddressSize;

#define INET_HaveLength  1

#define INET_accept      accept
#define INET_bind        bind
#define INET_close       close
#define INET_connect     connect
#define INET_error( sd ) errno
#define INET_getsockname getsockname
#define INET_listen      listen
#define INET_lookup      gethostbyname
#define INET_recv        recv
#define INET_recvfrom    recvfrom
#define INET_setsockopt  setsockopt
#define INET_socket      socket

#define INET_ioctl( sock, cmd, arg ) \
			 ioctl( (sock), (cmd), (int)arg )

#define INET_send( sock, data, size, flags ) \
			 send( (sock), (char *)(data), (size), (flags) )

#define INET_sendto( sock, data, size, flags, addr, addr_len ) \
			 sendto( (sock), (char *)(data), (size), (flags), \
				 (addr), (addr_len) )

#define INET_EALREADY    EALREADY
#define INET_EINPROGRESS EINPROGRESS
#define INET_EINTR       EINTR
#define INET_EISCONN     EISCONN
#define INET_EWOULDBLOCK EWOULDBLOCK

#if _BYTE_ORDER == _LITTLE_ENDIAN

    // The declaration of sockaddr_in in netinet/in.h was broken between
    // versions 1.0.0 and 1.0.1: the field
    //
    //     u_short sin_family;
    //
    // was replaced by the pair of fields
    //
    //     u_char sin_len;
    //     u_char sin_family;
    //
    // However, the kernel behaves as if it uses the old declaration.
    // This doesn't matter if _BYTE_ORDER == _BIG_ENDIAN, but i960 and
    // x86 processors have those bytes swapped.
    // We adapt by using the sin_len field instead of sin_family.

#   define sin_family sin_len

#endif

#endif // __RTtcp_h__
