#! perl -w
#	/****************************************************************/
#	/*                                                              */
#	/*  IBM Confidential                                            */
#	/*  OCO Source Materials                                        */
#	/*  [Component IBM Rational ROSE_RT TOOLSET : ROSE_RT]          */
#	/*   Copyright IBM Corp. [1998], [2004] .All Rights Reserved.  */
#	/*                                                              */
#	/*  The source code for this program is not published or        */
#	/*  or otherwise divested of its trade secrets, irrespective    */
#	/*  of what has been deposited with the U.S. Copyright Office.  */
#	/*                                                              */
#	/****************************************************************/
# %Copyright%

package rterror;

# Tags look like this

$startpattern   = '/\* {{{RME (.*) \*/$';
$endpattern     = '/\* }}}RME \*/$';
$ignorepattern1 = '/\* {{{USR \*/$';
$ignorepattern2 = '/\* }}}USR \*/$';

$enterstartpattern   = '/\* {{{RME enter(.*) \*/$';

%qualifies =
    (
	'associationEnd' => 'classifier',
	'capsuleRole'    => 'classifier',
	'choicePoint'    => 'classifier',
	'classAttribute' => 'classifier',
	'classifier'     => '.',
	'component'      => '.',
	'connector'      => 'classifier',
	'enter'          => 'classifier',
	'exit'           => 'classifier',
	'general'        => '*',
	'guard'          => 'transition',
	'inSignal'       => 'classifier',
	'operation'      => 'classifier',
	'outSignal'      => 'classifier',
	'package'        => '.',
	'parameter'      => '*',
	'port'           => 'classifier',
	'property'       => '*',
	'state'          => 'classifier',
	'tool'           => '*',
	'transition'     => 'classifier',
	'uniqueId'       => 'uniqueId'
    );

# print_error - Subroutine to print out an error
# Arguments
#   <object>    object of error
#   <message>   string containing the message
#   <level>     level giving the severity
#       0 = error
#       1 = warning

$maxError   = 2;
$worstError = $maxError;

@levelName = ( 'error', 'warning' );

sub print_error($$$)
{
    my( $object, $msg, $level ) = @_;

    print 'GES';
    print ' ', $object if $object;

    $msg =~ s/'/''/g;

    print " description '", $msg,
	  "' severity '",   $levelName[ $level ],
	  "'\n";

    $worstError = $level if $worstError > $level;
}

sub handle_errors()
{
    my $error;

    error_loop: foreach $error ( @main::errors )
    {
	chomp $error;

	foreach $handler ( @config::handlers )
	{
	    ( $pattern, $action ) = @$handler;

	    if( $error =~ /$pattern/ )
	    {
		&$action;
		next error_loop;
	    }
	}

	# not handled
	# &print_error( '', $error, 1 );
    }
}

# Handlers

sub action_ignore
{
}

sub action_print($$$$)
{
    my( $file, $lineno, $msg, $level ) = @_;

    if( open FILE, $file )
    {
	# found file directly
    }
    elsif( $main::srcdir && open FILE, "$main::srcdir/$file" )
    {
	# found file in source directory
    }
    else
    {
	&print_error( '', "$file:$lineno - $msg", $level );
	return;
    }

    my $object           = '';
    my $gesmsg           = '';
    my @taglineno        = ();
    my @taginfo          = ();
    my @afterstartlineno = 0;
    my $filelineno       = 0;
    my $fileline;
    my $isCapsule        = 0;
    my $capsString	 = '';
	my $USRtaglineno = 0;

    if( $file =~ m{^.*/(.*)\.c$} )
    {
	$capsString = $1;
    }

    # If we find this in the file, it is a capsule and not a passive class.
    $capsString .= "_InstanceData";

    while( $fileline = <FILE> )
    {
	++$filelineno;
	chomp $fileline;

	if( $fileline =~ m{$capsString} )
	{
	    $isCapsule = 1;
	}

	if( $filelineno == $lineno )
	{
	    if( $#taglineno >= 0 )
	    {
		$lineno -= $taglineno[$#taglineno];

		my $nextfileline = <FILE>;
		if( $nextfileline =~ m{$endpattern}o )
		{
		    # Error was reported on the line with the closing brace.
		    # Subtract one to put the error inside the function block.
		    $lineno--;
		}

		my %tags    = ();
		my $want    = '*';
		my $tagtext = '';

		while( $tagtext = pop @taginfo )
		{
		    while( $tagtext =~ s/\s*(\w+)\s+'([^']+)'\s*$// )
		    {
			my( $key, $value ) = ( $1, $2 );

			# only consider this key if we need it
			next if $want ne '*' && $key ne $want;

			my $next = $qualifies{$key};

			# skip unrecognized keys
			next unless $next;

			$want = $next;

			if( $key eq 'classifier' )
			{
			    if( $value !~ m{::} )
			    {
				# not fully qualified: keep looking
				$want = $key;
			    }

			    $value .= '::' . $tags{$key} if $tags{$key};
			}

			$tags{$key} = $value;
		    }

		    last if $want eq '.';
		}

		if ( $USRtaglineno > 0 )
		{
			$tags{'line'} = $lineno - $USRtaglineno + $taglineno[$#taglineno];
		}
		else
		{
			$tags{'line'} = $lineno ;
		}

		my @list = ();

		foreach $key ( sort keys %tags )
		{
		    push @list, $key, "'$tags{$key}'";
		}

		$object = join( ' ', @list );
		$gesmsg = $msg;
	    }
	    last;
	}
	elsif( $fileline =~ m{$startpattern}o )
	{
	    push @taginfo,   $1;
	    push @taglineno, $filelineno;

	    # This is used to keep track of braces that are right after the
	    # start RME tag. The openin brace can be either on the following
	    # line or the line after that.
	    # Note that entry code for a state in a passive class diagram is
	    # special, because the user code doesn't come until after the second
	    # opening brace and that brace comes four lines after the tag.

	    if( ! $isCapsule && $fileline =~ $enterstartpattern )
	    {
		push @afterstartlineno, 4;
	    }
	    else
	    {
		push @afterstartlineno, 2;
	    }
	}
	elsif( $fileline =~ m{$endpattern}o )
	{
	    pop @taglineno;
	    pop @taginfo;
	    pop @afterstartlineno;
	}
	elsif( $fileline =~ m{$ignorepattern1}o
	    or $fileline =~ m{$ignorepattern2}o )
	{
		if ( $fileline =~ m{$ignorepattern1}o )
		{
			$USRtaglineno = $filelineno;
		}
		elsif ( $fileline =~ m{$ignorepattern2}o )
		{
			$USRtaglineno = 0;
		}
	    ++$taglineno[$#taglineno] if $#taglineno >= 0;
	    --$afterstartlineno[$#afterstartlineno] if $#afterstartlineno >= 0 && $afterstartlineno[$#afterstartlineno] > 0;
	}
	elsif( $fileline =~ m{\s*\{$} ||
	       $fileline =~ m{if\($} )  # Choice points in passive class states start like this.
	{
	    $taglineno[$#taglineno] = $filelineno if $#taglineno >= 0 && $#afterstartlineno >= 0 && $afterstartlineno[$#afterstartlineno] > 0;

	    if( ! $isCapsule && 
		$#taginfo >= 0 && $taginfo[ $#taginfo ] =~ "^enter .*" && 
	        $#afterstartlineno >= 0 && $afterstartlineno[$#afterstartlineno] > 2 )
	    {
		# Remember, entry actions for passive class states are different
		# - see above. 
		--$afterstartlineno[$#afterstartlineno];
	    }
	    else
	    {
		$afterstartlineno[$#afterstartlineno + 1] = 0;
	    }
	}
	else
	{
	    --$afterstartlineno[$#afterstartlineno] if $#afterstartlineno >= 0 && $afterstartlineno[$#afterstartlineno] > 0;
	}
    }

    close FILE;

    $gesmsg = "$file:$lineno - $msg" unless $gesmsg;

    &print_error( $object, $gesmsg, $level );
}

sub action_message($$)
{
    my( $msg, $level ) = @_;

    if( $main::component ne '' )
    {
	&print_error( "component '$main::component'", $msg, $level );
    }
    else
    {
	&print_error( '', $msg, $level );
    }
}

return 1;
